function [f, b] = SysChangeOfVar(x, M, C, G, F)
% SysChangeOfVar gives the system matrices f and b a system represented 
% with notation dx/dt = dx = x':
%
%       x'=f(x)+b(x)u.
%
% Where:
%       x = [x(1) x(2) ... x(2n)] 
%         = [q(1) q(2) ... q(n) dq(1) dq(2) ... dq(n)]
%         = [q(1) q(2) ... q(n) q'(1) q'(2) ... q'(n)]
%
% When the original system state representation is given by:
%       M(q)q''+C(q,q')q'+G(q)=F(q)u
%
% q is the original system degrees of freedom.
%
% Written by: Jrgen Herje Nilsen
% Date: 24.02.2014
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[rowq, columnq] = size(x);
if columnq ~= 1 && rowq ~= 0
    disp('state vector consist of mutiple columns')
    return;    
else
    if columnq ~= 1 && rowq == 1
        x = x'; % if q is row vector, make q column vector
    end
    f  = zeros(length(x),1);
    b  = zeros(length(x),1);
    
    f(1:length(x)/2,1) = x(length(x)/2+1:end,1);
    
    %Make the row vector C(q,q')*x'
    [~, columnC] = size(C);
    Cq = C;
    if columnC > 1 
        Cq = C*[x(3) x(4)]';
    end
        
    CG = -(Cq+G);
    f(length(x)/2+1:end,1) = M\CG;
    b(length(x)/2+1:end,1) = M\F;
end