% Defining parameters needed, where:
% l is length of first link (height of bike)
% rb is distance from first joint to mass center of the first link (bike)
% rp is the distance from second joint to mass center of the second link
% mb is the mass of the mass center of first link (bike)
% mp is the mass of the mass center of second link (pendulum)
% thetab is the tilt angle of first link relative to initial coord syst.
% thetap is the tilt angle of second link relative to initial coord syst.
% g is the acceleration due to gravity.
% taum is the torque provided from the motor balancing the pendulum.
l = 0.72;    % [m] approximately
rb = 0.4;    % [m] approximately
rp = 0.2;    % [m] approximately
mb = 31.118; % [kg] approximately
mp = 5;      % [kg] approximately
taum = 0;    % [?] torque from DC-motor
g = 9.81;    % [m/s^2] gravity acceleration
I_b = 0;     % [Nm] Moment of Inertia of bicycle
I_p = 0;     % [Nm] Moment of Inertia of pendulum

% l = 1;
% rb = 0.5;
% rp = 1;
% mb = 1;
% mp = 1;
% g = 9.81;
% I_b = 0.083;  % [Nm^2]
% I_p = 0.33;   % [Nm^2]



% Friction Coefficients:
% k_fb = 0.1; % Bicycle friction coefficient. In real life equal to zero 
% k_fp = 0.5; % Pendulum motor friction coefficient. In real life very low, maybe neglect?
k_fb = 0;
k_fp = 0;

% Feedback Control Coefficients:
% k_p = 1;
% k_d = 1;

% Linear Model around the unstable sadle equilibrium thetab = thetap = 0

M_lin = [I_b+I_p+mb*rb^2+mp*(l^2+rp^2+2*rp*l) I_p+mp*rp*(l+rp) ;
           I_p+mp*rp*(l+rp)                  I_p+mp*rp^2  ];
    
C_lin = zeros(2,2);

F_lin = [0; 1];

dGdq = [-g*(mb*rb+mp*rp+mp*l) -mp*rp*g;
             -mp*rp*g         -mp*rp*g]; 
    
A = [     zeros(2,2)         eye(2);
     -inv(M_lin)*dGdq  -inv(M_lin)*C_lin];

B = [  zeros(2,1);
     inv(M_lin)*F_lin ];
 
%C = eye(4); % Must have eye(4), else P is inf.

C_lin = [1 0 0 0;
         0 1 0 0];

CO = ctrb(A,B);
OB = obsv(A,sqrt(C_lin));


% Choosing wighted matrices Q and R:

%Q = C'*C;
Q = 0.1*[1 0 0 0;0 1 0 0; 0 0 1 0; 0 0 0 1];
% Q = diag([20 100 2 10]);

% R_weight = 1e5;
R_weight = 1;
R = R_weight*eye(1);


[K,P,EIG] = lqr(A,B,Q,R);
%K1 = inv(R)*B'*P;


% Symmetric pos. def. soln of the algebraic Riccati equation: P

%P = inv(C*inv(B*K-A)*B);




